import sys.io.File;
//import highlighter.Highlighter; //http://lib.haxe.org/p/html-haxe-code-highlighter/ //download css + js instead
using StringTools;


class SiteGen {
	public static function main() {
		doIndex();
		doNews();
		doUML();
		doExamples();
	}
	
	
	static function doIndex() {
		var html = header("Home");
		html += mdToW3Html(Xml.parse(embed("h1", "article", Markdown.markdownToHtml(File.getContent("README.md")))));
		html += footer();
		File.saveContent("doc/web/index.html", html);
	}
	
	
	static function doNews() {
		var html = header("News");
		html += mdToW3Html(Xml.parse(embed("h1", "article", Markdown.markdownToHtml(File.getContent("doc/ChangeLog.md")))));
		html += footer();
		File.saveContent("doc/web/news.html", html);
	}
	
	
	static function doUML() {
		var html = header("UML");
		var mdContent = 'Containers\n==========\n\n' +
						'![UML container classes](http://haxe.tuxfamily.org/buw/png/uml1-containers.png "UML container classes")\n\n\n' +
						'Base widgets\n============\n\n' +
						'![UML base widgets](http://haxe.tuxfamily.org/buw/png/uml2-base-widgets.png "UML base widgets")\n\n\n' +
						'Advanced controls\n=================\n\n' +
						'![UML advanced controls](http://haxe.tuxfamily.org/buw/png/uml3-advanced-controls.png "UML advanced controls")\n';
		html += mdToW3Html(Xml.parse(embed("h1", "article", Markdown.markdownToHtml(mdContent))));
		html += footer();
		File.saveContent("doc/web/uml.html", html);
	}
	
	
	static function doExamples() {
		var html = header("Examples");
		var section = "";
		for (i in 1...6) {
			section += '<h1>Example $i</h1>\n';
			section += '<h2>Screenshot</h2>\n<p><img src="png/example$i.png" alt="Example $i screenshot"/></p>\n';
			section += "<h2>Source code</h2>\n<pre><code>" + File.getContent('samples/Source/Example$i.hx').replace("<", "&lt;") + "</code></pre>\n";
		}
		html += mdToW3Html(Xml.parse(embed("h1", "article", section))); //.replace("&lt;", "<");
		html += footer();
		File.saveContent("doc/web/examples.html", html);
	}
	
	
	static function mdToW3Html (xml : Xml) : String {
		for (article in xml.elementsNamed("article")) {
			article.set("style", "padding-top:3em");
			article.set("id", article.firstChild().firstChild().nodeValue.toLowerCase()); //article->h1
			article.firstChild().set("class", "w3-theme");
			for (tag in [ "h2", "p", "ul" ]) {
				for (t in article.elementsNamed(tag)) {
					t.set("class", "w3-text-theme");
				}
			}
			for (pre in article.elementsNamed("pre")) {
				pre.set("class", "w3-theme-light"); //w3-code
				pre.set("style", "overflow:auto");
				if (pre.firstChild().firstChild().nodeValue.indexOf("class") != -1) {
					pre.firstChild().set("class", "prettyprint haxe");
					pre.firstChild().set("style", "margin:0; padding:0");
				}
			}
			for (p in article.elementsNamed("p")) {
				for (img in p.elementsNamed("img")) {
					img.set("class", "w3-image");
				}
			}
		}
		return xml.toString().replace("\t", "  ");
	}
	
	
	static function embed(embededTag : String, outerTag : String, content : String) : String {
		var contentParts = content.split("<" + embededTag + ">");
		contentParts[0] = "<" + outerTag + ">";
		for (i in 1...contentParts.length-1) {
			contentParts[i] += "</" + outerTag + ">" + "<" + outerTag + ">";
		}
		contentParts[contentParts.length-1] += "</" + outerTag + ">";
		return contentParts.join("<" + embededTag + ">");
	}
	
	
	static function header(title : String) : String {
		var titlelc = title.toLowerCase();
		return	'<!DOCTYPE html>\n' +
				'<html lang="en"><head>\n' +
				'	<title>Basic and Usefull Widgets - $title</title>\n' +
				'	<meta charset="UTF-8"/>\n' +
				'		<link rel="stylesheet" href="https://www.w3schools.com/w3css/4/w3.css"/>\n' +
				'		<link rel="stylesheet" href="https://www.w3schools.com/lib/w3-theme-blue-grey.css">\n' +
				'		<meta name="viewport" content="width=device-width, initial-scale=1">\n' +
				'		<script>\n' +
				'function toggleMobileMenu() {\n' +
				'	var m = document.getElementById("mobileNav");\n' +
				'	if (m.className.indexOf("w3-show") == -1) {\n' +
				'		m.className += " w3-show";\n' +
				'	} else {\n' +
				'		m.className = m.className.replace(" w3-show", "");\n' +
				'	}\n' +
				'}\n' +
				'</script>\n' +
				'</head><body>\n' +
				'	<header>\n' +
				'		<nav class="w3-theme w3-top w3-bar"><div>\n' +
				'			<a id="home" href="index.html#introduction" class="w3-bar-item w3-button w3-hide-small">Introduction</a>\n' +
				'			<a href="index.html#usage" class="w3-bar-item w3-button w3-hide-small">Usage</a>\n' +
				'			<a id="news" href="news.html" class="w3-bar-item w3-button w3-hide-small">News</a>\n' +
				'			<a id="examples" href="examples.html" class="w3-bar-item w3-button w3-hide-small">Examples</a>\n' +
				'			<a id="uml" href="uml.html" class="w3-bar-item w3-button w3-hide-small">UML</a>\n' +
				'			<a target="_blank" href="api/index.html" class="w3-bar-item w3-button w3-hide-small">API</a>\n' +
				'			<a target="_blank" href="http://download.tuxfamily.org/haxe/buw" class="w3-bar-item w3-button w3-hide-small">Download</a>\n' +
				'			<a href="index.html#introduction" class="w3-bar-item w3-button w3-hide-large w3-hide-medium">Basic and Usefull Widgets</a>\n' +
				'			<a class="w3-bar-item w3-button w3-hover-black w3-right w3-hide-medium w3-hide-large" onclick="toggleMobileMenu(); return false;" title="Menu">&#9776;</a>\n' +
				'		</div>\n' +
				'		<div id="mobileNav" class="w3-theme w3-bar-block w3-hide">\n' +
				'			<a href="index.html#usage" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">Usage</a>\n' +
				'			<a href="news.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">News</a>\n' +
				'			<a href="examples.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">Examples</a>\n' +
				'			<a href="uml.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">UML</a>\n' +
				'			<a href="api/index.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">API</a>\n' +
				'			<a href="http://download.tuxfamily.org/haxe/buw" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">Download</a>\n' +
				'		</div></nav>\n' +
				'		<h1 class="w3-center w3-border w3-text-theme w3-hide-small" style="margin-top:3em; padding:0.5em">Basic and Usefull Widgets - $title</h1>\n' +
				'		<script>document.getElementById("$titlelc").className += " w3-theme-l1";</script>\n' +
				'	</header><section>';
	}
	
	
	static function footer() : String {
		return	'\n	</section>\n' +
				'	<footer class="w3-theme w3-container">frank@fr.endres</footer>\n' +
				'	<link rel="stylesheet" href="highlighter.min.css" /><script src="highlighter.js"></script>\n' +
				'</body></html>';
	}
}
