import buw.*;

/**
 * example 3: list view
 * using: VBox, ListView, Separator, Label & TextButton
 */

typedef Book = {
	var author : String;
	var title : String;
}

class Example3 extends openfl.display.Sprite {
	//a list view is a usefull widget to display collections of items; it is strongly typed:
	var booksList : ListView<Book>;

	function new() {
		super();
		var rootWindow = new VBox();
		rootWindow.pack(new TextButton(onAddBooksClick, "Add books"));
		Screen.display(rootWindow); //enables vertical scrolling if needed
		rootWindow.pack(new Separator());

		//to create a list view, one must tell how to render an item, and eventually tell what to do when an item is clicked:
		booksList = new ListView(renderBook, onBookClick);
		rootWindow.pack(booksList);
	}

	function onAddBooksClick(w : TextButton) {
		//list views source can be arrays or lists:
		var books : Array<Book> = new Array();
		books.push({title : "Les misérables", author : "Victor Hugo"});
		books.push({title : "20 000 lieues sous les mers", author : "Jules Verne"});
		
		//first time the button is clicked, source is set:
		if (booksList.source == null) {
			booksList.source = books;
		} else {
			//then, for demonstration purpose, the same books are added to the existing source:
			for (b in books) {
				booksList.push(b);
			}
		}
	}

	//the render function is given an item as a parameter; the returned widget will be displayed by the list view;
	//this function is called once per widget in the item collection:
	function renderBook (b : Book) : Widget {
		//the returned widget is here a label:
		return new Label(b.title + " - " + b.author);
		//it could be something more complex like an horizontal box with an image and a label for example
	}

	//the callback function is givent the clicked item as parameter:
	function onBookClick(b : Book) {
		trace(b.title);
	}
}
